/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jsp.taglib.beans;

import com.squeakysand.commons.beans.BeanHelper;
import com.squeakysand.commons.beans.BeanHelperException;
import com.squeakysand.commons.beans.PropertyDescriptorFilter;
import com.squeakysand.jsp.tagext.BodyContent;
import com.squeakysand.jsp.tagext.EnhancedSimpleTagSupport;
import com.squeakysand.jsp.tagext.VariableScope;
import com.squeakysand.jsp.tagext.annotations.JspTag;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;
import com.squeakysand.jsp.tagext.annotations.JspTagVariable;

import java.beans.PropertyDescriptor;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.List;

import javax.servlet.jsp.JspException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Iterates over the readable JavaBean properties of the supplied bean and injects the name and value of each property into the body of the tag.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
@JspTag(bodyContent = BodyContent.SCRIPTLESS, example = "<ss:forEachBeanProperty bean=\"${myBean}\">${name} - ${value}<br/></ss:forEachBeanProperty>")
public class ForEachBeanPropertyTag extends EnhancedSimpleTagSupport {

    /**
     * The name of the current bean property.
     */
    @JspTagVariable(scope = VariableScope.NESTED)
    private static final String NAME_VARIABLE_NAME = "name";

    /**
     * The value of the current bean property.
     */
    @JspTagVariable(scope = VariableScope.NESTED, variableClass = Object.class)
    private static final String VALUE_VARIABLE_NAME = "value";

    private static Logger LOG = LoggerFactory.getLogger(ForEachBeanPropertyTag.class);

    private Object bean;

    @Override
    public void doTag() throws JspException, IOException {
        validateNotNullAttribute("bean", bean);
        List<PropertyDescriptor> properties = null;
        try {
            properties = BeanHelper.getPropertyDescriptors(bean.getClass(), PropertyDescriptorFilter.Include.READABLE);
        } catch (BeanHelperException e) {
            LOG.error(e.getMessage(), e);
        }
        if (properties != null) {
            for (PropertyDescriptor property : properties) {
                String name = property.getName();
                Object value = null;
                try {
                    value = property.getReadMethod().invoke(bean, (Object[]) null);
                } catch (IllegalAccessException e) {
                    LOG.error(e.getMessage(), e);
                    continue;
                } catch (IllegalArgumentException e) {
                    LOG.error(e.getMessage(), e);
                    continue;
                } catch (InvocationTargetException e) {
                    LOG.error(e.getMessage(), e);
                    continue;
                }
                setPageAttribute(NAME_VARIABLE_NAME, name);
                setPageAttribute(VALUE_VARIABLE_NAME, value);
                getJspBody().invoke(null);
            }
        }
    }

    public Object getBean() {
        return bean;
    }

    /**
     * The bean that properties should be iterated over.
     * 
     * @param bean a Java Object that will be treated as a JavaBean.
     */
    @JspTagAttribute(required = true, rtexprvalue = true)
    public void setBean(Object bean) {
        this.bean = bean;
    }

}
