/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jsp.taglib.logging;

import com.squeakysand.commons.lang.StringUtils;
import com.squeakysand.jsp.tagext.EnhancedSimpleTagSupport;
import com.squeakysand.jsp.tagext.annotations.JspTag;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Logs a message from within a JSP page using the SLF4J framework.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
@JspTag(name = "log")
public class LoggerTag extends EnhancedSimpleTagSupport {

    private static final Logger LOG = LoggerFactory.getLogger(LoggerTag.class);

    private Level level;
    private String logName;
    private String message;

    @Override
    public void doTag() {
        Logger logger = null;
        // first check if there was a logger name set on this tag
        if (StringUtils.isNotNullOrEmpty(logName)) {
            logger = LoggerFactory.getLogger(logName);
        } else {
            // else check to see if there was a logger name set at the page level
            logName = (String) getPageAttribute(SetLoggerNameTag.LOGGER_NAME_ATTRIBUTE);
            if (StringUtils.isNotNullOrEmpty(logName)) {
                logger = LoggerFactory.getLogger(logName);
            } else {
                // else use the logger of this tag class
                logger = LOG;
            }
        }
        // first check if there was a level set on this tag
        if (level == null) {
            // check to see if there was a logger level set at the page level
            level = (Level) getPageAttribute(SetLoggerLevelTag.LOGGER_LEVEL_ATTRIBUTE);
            if (level == null) {
                // no level set anywhere, so use default level
                level = Level.INFO;
            }
        }
        if (LOG.isDebugEnabled()) {
            LOG.debug("using logger: {} and level: {}", logger.getName(), level);
        }
        switch (level) {
        case TRACE:
            logger.trace(message);
            break;
        case DEBUG:
            logger.debug(message);
            break;
        case INFO:
            logger.info(message);
            break;
        case WARN:
            logger.warn(message);
            break;
        case ERROR:
            logger.error(message);
            break;
        default:
            throw new IllegalStateException(String.format("unknown logging level found: %s", level));
        }
    }

    /**
     * Accessor for the <code>level</code> property.
     * 
     * @return the current value of the <code>level</code> property.
     */
    public Level getLevel() {
        return level;
    }

    /**
     * Accessor for the <code>logName</code> property.
     * 
     * @return the current value of the <code>logName</code> property.
     */
    public String getLogName() {
        return logName;
    }

    /**
     * Accessor for the <code>message</code> property.
     * 
     * @return the current value of the <code>message</code> property.
     */
    public String getMessage() {
        return message;
    }

    /**
     * The logging level to use. If no value is provided for this attribute, then the tag will try to find a {@link Level} set by the <code>setLogLevel</code>
     * tag earlier in the page, if it cannot find a {@link Level} that way, then it will default to the <code>INFO</code> level.
     * 
     * @param level the level to log at.
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setLevel(Level level) {
        this.level = level;
    }

    /**
     * The name of the logger to use. If no value is provided for this attribute, then the tag will try to find a name set by the <code>setLogName</code> tag
     * earlier in the page, if it cannot find a name that way, it will default to using the built in logger of this class.
     * 
     * @param logName the name of the logger to use.
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setLogName(String logName) {
        this.logName = logName;
    }

    /**
     * The message to be logged.
     * 
     * @param message the message to be logged.
     */
    @JspTagAttribute(required = true, rtexprvalue = true)
    public void setMessage(String message) {
        this.message = message;
    }

}
