/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jsp.taglib.text;

import com.squeakysand.commons.lang.ToStringHelper;
import com.squeakysand.jsp.tagext.annotations.JspElFunction;

import org.apache.commons.lang.StringEscapeUtils;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A collection of functions for use in EL expressions related to manipulating text.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
public class Functions {

    private static final Logger LOG = LoggerFactory.getLogger(Functions.class);

    /**
     * Escapes plain text to be HTML safe.
     * 
     * @param unescapedHtml plain text to be escaped.
     * @return the passed in HTML with any special characters replaced with their HTML equivalents.
     */
    @JspElFunction
    public static String escapeHtml(String unescapedHtml) {
        String result = StringEscapeUtils.escapeHtml(unescapedHtml);
        LOG.debug("\"{}\" escaped to \"{}\"", unescapedHtml, result);
        return result;
    }

    /**
     * Returns the text content between HTML markup.
     * 
     * @param html the target HTML snippet.
     * @return the text content of the HTML snippet.
     */
    @JspElFunction
    public static String removeMarkup(String html) {
        Document document = Jsoup.parse(html);
        String result = document.text();
        LOG.debug("\"{}\" became \"{}\"", html, result);
        return result;
    }

    /**
     * Removes all HTML markup from a string and also replaces any XML entity references with their plain text equivalents.
     * 
     * @param htmlString a string containing HTML markup and entities.
     * 
     * @return plain text string with HTML markup removed and entities replaced with plain text characters.
     */
    @JspElFunction
    public static String stripHtml(String htmlString) {
        String result = removeMarkup(htmlString);
        result = unescapeHtml(result);
        LOG.debug("\"{}\" stripped down to \"{}\"", htmlString, result);
        return result;
    }

    /**
     * Generates a {@link String} representation of the specified object.
     * 
     * @param o the target object.
     * @return a {@link String} representation of the specified object.
     */
    @JspElFunction
    public static String toString(Object o) {
        return ToStringHelper.toString(o);
    }

    /**
     * Removes HTML/XML escape sequences and replaces them with their plain text equivalents.
     * 
     * @param escapedHtml the target HTML snippet.
     * @return the target HTML with all HTML/XML entities replaced with their plain text equivalents.
     */
    @JspElFunction
    public static String unescapeHtml(String escapedHtml) {
        String result = StringEscapeUtils.unescapeHtml(escapedHtml);
        LOG.debug("\"{}\" unescaped to \"{}\"", escapedHtml, result);
        return result;
    }

}
