/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jsp.taglib.util;

import com.squeakysand.commons.lang.MathUtils;
import com.squeakysand.commons.lang.ToStringHelper;
import com.squeakysand.jsp.tagext.EnhancedSimpleTagSupport;
import com.squeakysand.jsp.tagext.VariableScope;
import com.squeakysand.jsp.tagext.annotations.JspTag;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;
import com.squeakysand.jsp.tagext.annotations.JspTagVariable;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Generates a random string and injects it into the JSP page context.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
@JspTag(name = "randomString")
public class RandomStringGeneratorTag extends EnhancedSimpleTagSupport {

    private static final String LOWERCASE_LETTERS = "abcdefghijklmnopqrstuvwxyz";

    private static final String UPPERCASE_LETTERS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

    private static final String NUMBERS = "0123456789";

    private static final Integer DEFAULT_EXACT_LENGTH = 10;

    private static final Integer DEFAULT_MIN_LENGTH = 1;

    private static final Logger LOG = LoggerFactory.getLogger(RandomStringGeneratorTag.class);

    private String prefix;
    private String suffix;
    private String extraChars;
    private Integer maxLength;
    private Integer minLength;
    private Integer exactLength;
    private RandomStringMode mode;

    private String var;

    @Override
    public void doTag() throws JspException, IOException {
        if (LOG.isDebugEnabled()) {
            LOG.debug("tag configured as: {}", ToStringHelper.toString(this));
        }
        String sourceChars = getSourceCharacters();
        Integer resultLength = getResultLength();
        String result = generateResult(sourceChars, resultLength);
        if (prefix != null) {
            result = prefix + result;
        }
        if (suffix != null) {
            result += suffix;
        }
        setPageAttribute(var, result);
    }

    /**
     * Accessor for the <code>exactLength</code> property.
     * 
     * @return the current value of the <code>exactLength</code> property.
     */
    public Integer getExactLength() {
        Integer result = exactLength;
        if (result == null) {
            result = DEFAULT_EXACT_LENGTH;
        }
        return result;
    }

    /**
     * Accessor for the <code>extraChars</code> property.
     * 
     * @return the current value of the <code>extraChars</code> property.
     */
    public String getExtraChars() {
        return extraChars;
    }

    /**
     * Accessor for the <code>maxLength</code> property.
     * 
     * @return the current value of the <code>maxLength</code> property.
     */
    public Integer getMaxLength() {
        return maxLength;
    }

    /**
     * Accessor for the <code>minLength</code> property.
     * 
     * @return the current value of the <code>minLength</code> property.
     */
    public Integer getMinLength() {
        Integer result = minLength;
        if (result == null) {
            result = DEFAULT_MIN_LENGTH;
        }
        return result;
    }

    /**
     * Accessor for the <code>mode</code> property.
     * 
     * @return the current value of the <code>mode</code> property.
     */
    public RandomStringMode getMode() {
        RandomStringMode result = mode;
        if (result == null) {
            result = RandomStringMode.LETTERS;
        }
        return result;
    }

    /**
     * Accessor for the <code>prefix</code> property.
     * 
     * @return the current value of the <code>prefix</code> property.
     */
    public String getPrefix() {
        return prefix;
    }

    /**
     * Accessor for the <code>suffix</code> property.
     * 
     * @return the current value of the <code>suffix</code> property.
     */
    public String getSuffix() {
        return suffix;
    }

    /**
     * Accessor for the <code>var</code> property.
     * 
     * @return the current value of the <code>var</code> property.
     */
    public String getVar() {
        return var;
    }

    /**
     * The required exact length of the random {@link String} to generate.
     * 
     * @param exactLength the exact length to use.
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setExactLength(Integer exactLength) {
        this.exactLength = exactLength;
    }

    /**
     * Extra characters to include in the generation process.
     * 
     * @param extraChars s {@link String} of characters that can be used in addition to the normal characters when generating the random {@link String}.
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setExtraChars(String extraChars) {
        this.extraChars = extraChars;
    }

    /**
     * The allowed maximum length of the generated {@link String}.
     * 
     * @param maxLength the maximum length to use.
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setMaxLength(Integer maxLength) {
        this.maxLength = maxLength;
    }

    /**
     * The allowed minimum length of the generated {@link String}.
     * 
     * @param minLength the minimum length to use.
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setMinLength(Integer minLength) {
        this.minLength = minLength;
    }

    /**
     * The random generation mode to use. Valid values are:
     * 
     * <ul>
     * <li>LETTERS</li>
     * <li>NUMBERS</li>
     * <li>LETTERS_AND_NUMBERS</li>
     * <li>LOWERCASE_LETTERS</li>
     * <li>UPPERCASE_LETTERS</li>
     * <li>LOWERCASE_LETTERS_AND_NUMBERS</li>
     * <li>UPPERCASE_LETTERS_AND_NUMBERS</li>
     * </ul>
     * 
     * @param mode the mode to use.
     * 
     * @see RandomStringMode
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setMode(RandomStringMode mode) {
        this.mode = mode;
    }

    @JspTagAttribute(rtexprvalue = true)
    public void setPrefix(String prefix) {
        this.prefix = prefix;
    }

    /**
     * Appended to the end of the random string.
     * 
     * @param suffix
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setSuffix(String suffix) {
        this.suffix = suffix;
    }

    /**
     * The name of the context variable to inject into the page.
     * 
     * @param var the name of the context variable to inject into the page.
     */
    @JspTagVariable(scope = VariableScope.AT_END)
    @JspTagAttribute(required = true)
    public void setVar(String var) {
        this.var = var;
    }

    private String generateResult(String sourceChars, Integer resultLength) {
        char[] chars = new char[resultLength];
        int sourceCharsLength = sourceChars.length();
        for (int i = 0; i < resultLength; i++) {
            int sourceCharIndex = MathUtils.random(0, sourceCharsLength - 1);
            chars[i] = sourceChars.charAt(sourceCharIndex);
        }
        LOG.debug("result: {}", chars);
        return new String(chars);
    }

    private Integer getResultLength() {
        Integer result = null;
        if (getMaxLength() != null) {
            Integer min = getMinLength();
            Integer max = getMaxLength();
            result = MathUtils.random(min, max);
        } else {
            result = getExactLength();
        }
        LOG.debug("result length: {}", result);
        return result;
    }

    private String getSourceCharacters() {
        String result = null;
        switch (getMode()) {
        case LETTERS:
            result = LOWERCASE_LETTERS + UPPERCASE_LETTERS;
            break;
        case NUMBERS:
            result = NUMBERS;
            break;
        case LETTERS_AND_NUMBERS:
            result = LOWERCASE_LETTERS + UPPERCASE_LETTERS + NUMBERS;
            break;
        case LOWERCASE_LETTERS:
            result = LOWERCASE_LETTERS;
            break;
        case UPPERCASE_LETTERS:
            result = UPPERCASE_LETTERS;
            break;
        case LOWERCASE_LETTERS_AND_NUMBERS:
            result = LOWERCASE_LETTERS + NUMBERS;
            break;
        case UPPERCASE_LETTERS_AND_NUMBERS:
            result = UPPERCASE_LETTERS + NUMBERS;
            break;
        }
        if (extraChars != null) {
            result += extraChars;
        }
        return result;
    }

}
